﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Reflection;

namespace System.Collections.Generic
{
  public static class IEnumerableExtension
  {
    public static DataTable ToDataTable<T>(this IEnumerable<T> enumerable)
    {
      return ToDataTable(enumerable, null);
    }
    public static DataTable ToDataTable<T>(this IEnumerable<T> enumerable, Func<IEnumerable<PropertyInfo>, IEnumerable<PropertyInfo>> filterPropertiesDelegate)
    {
      DataTable table = new DataTable();
      IEnumerable<PropertyInfo> props = typeof(T).GetProperties();
      if (filterPropertiesDelegate != null)
        props = filterPropertiesDelegate(props);
      foreach (PropertyInfo prop in props)
        table.Columns.Add(prop.Name, prop.PropertyType);
      foreach (T t in enumerable)
      {
        var row = table.NewRow();
        foreach (PropertyInfo prop in props)
          row[prop.Name] = prop.GetValue(t, null);
        table.Rows.Add(row);
      }
      return table;
    }
    public static IEnumerable<T> Distinct<T>(this IEnumerable<T> source, Func<T, T, bool> equalityComparer)
    {
        int sourceCount = source.Count();
        for (int i = 0; i < sourceCount; i++)
        {
            bool found = false;
            for (int j = 0; j < i; j++)
                if (equalityComparer(source.ElementAt(i), source.ElementAt(j))) // In some cases, it's better to convert source in List<T> (before first for)
                {
                    found = true;
                    break;
                }
            if (!found)
                yield return source.ElementAt(i);
        }
    }
  }
}
