﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Collections;

namespace System.Text.RegularExpressions
{
  public static class RegexExtension
  {
    public static int MaxLength(this Regex regex)
    {
      return MaxLength(regex.ToString());
    }

    private static BindingFlags _bindingFlags = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Static;
    private static Assembly _assembly = typeof(Regex).Assembly;

    /// <remarks>Usable only for "simples" patterns</remarks>
    public static int MaxLength(string pattern)
    {
      if (!(pattern.StartsWith("^") && pattern.EndsWith("$")))
        return int.MaxValue;

      Type t = _assembly.GetType("System.Text.RegularExpressions.RegexParser");
      object tree = t.GetMethod("Parse", _bindingFlags).Invoke(null, new object[] { pattern, RegexOptions.None });
      return NodeMaxLength(tree.GetType().GetField("_root", _bindingFlags).GetValue(tree));
    }

    private static int NodeMaxLength(object node)
    {
      int nodeType = (int)(_assembly.GetType("System.Text.RegularExpressions.RegexNode").GetField("_type", _bindingFlags).GetValue(node));
      if ((nodeType == (int)(node.GetType().GetField("Capture", _bindingFlags).GetValue(null))) ||
        (nodeType == (int)(node.GetType().GetField("Concatenate", _bindingFlags).GetValue(null))) ||
        (nodeType == (int)(node.GetType().GetField("Beginning", _bindingFlags).GetValue(null))) ||
        (nodeType == (int)(node.GetType().GetField("EndZ", _bindingFlags).GetValue(null))))
        return NodeMaxLength(node.GetType().GetField("_children", _bindingFlags).GetValue(node) as System.Collections.IList);
      else if (nodeType == (int)(node.GetType().GetField("Loop", _bindingFlags).GetValue(null)))
      {
        int max = (int)(node.GetType().GetField("_n", _bindingFlags).GetValue(node));
        if (max == int.MaxValue)
          return int.MaxValue;
        int childrenValue = NodeMaxLength(node.GetType().GetField("_children", _bindingFlags).GetValue(node) as IEnumerable);
        if (childrenValue == int.MaxValue)
          return int.MaxValue;
        return max * childrenValue;
      }
      else if ((nodeType == (int)(node.GetType().GetField("Set", _bindingFlags).GetValue(null))) ||
        (nodeType == (int)(node.GetType().GetField("One", _bindingFlags).GetValue(null))) ||
        (nodeType == (int)(node.GetType().GetField("Notone", _bindingFlags).GetValue(null))))
        return 1;
      else if (nodeType == (int)(node.GetType().GetField("Setloop", _bindingFlags).GetValue(null)))
        return (int)(node.GetType().GetField("_n", _bindingFlags).GetValue(node));
      else if (nodeType == (int)(node.GetType().GetField("Alternate", _bindingFlags).GetValue(null)))
      {
        int value = 0;
        foreach (object subNode in node.GetType().GetField("_children", _bindingFlags).GetValue(node) as IEnumerable)
        {
          int subValue = NodeMaxLength(subNode);
          if (subValue > value)
            value = subValue;
        }
        return value;
      }
      else if (nodeType == (int)(node.GetType().GetField("Multi", _bindingFlags).GetValue(null)))
        return ((string)(node.GetType().GetField("_str", _bindingFlags).GetValue(node))).Length;
      else
        throw new NotImplementedException();
    }
    private static int NodeMaxLength(IEnumerable nodes)
    {
      if (nodes == null)
        return 0;
      int value = 0;
      foreach (object node in nodes)
      {
        int nodeValue = NodeMaxLength(node);
        if (nodeValue == int.MaxValue)
          return int.MaxValue;
        value += nodeValue;
      }
      return value;
    }
  }
}
